from django.contrib import admin
from django import forms
from django.contrib.admin import AdminSite
from .forms import CustomAuthenticationForm, BaseAdminForm, MingoUserAddForm, MingoUserChangeForm
from django.contrib.admin import ModelAdmin
from django.urls import NoReverseMatch, Resolver404, resolve, reverse
from django.contrib.auth.admin import UserAdmin


#Change the default login form
AdminSite.login_form = CustomAuthenticationForm

#Change the default change form for model
ModelAdmin.form = BaseAdminForm

#Change the user adding form
UserAdmin.add_form = MingoUserAddForm
UserAdmin.form = MingoUserChangeForm


#Add class to related field like foreignkey select field
def change_formfield_for_foreignkey(original_method):
    def wrapper(self, db_field, request, **kwargs):
        kwargs["widget"] = forms.Select(attrs={"class": "form-select"})
        return original_method(self, db_field, request, **kwargs)
    return wrapper
ModelAdmin.formfield_for_foreignkey = change_formfield_for_foreignkey(ModelAdmin.formfield_for_foreignkey)









# In yourapp/widgets.py

from django.contrib.admin.widgets import AdminTextInputWidget

class MingoTextInputWidget(AdminTextInputWidget):
    def __init__(self, attrs=None):
        default_attrs = {
            'class': 'vTextField form-control',  # Add your custom classes here
        }
        if attrs:
            default_attrs.update(attrs)
        super().__init__(attrs=default_attrs)



from typing import Any
from django import forms
from django.contrib.auth.forms import AuthenticationForm, AdminPasswordChangeForm, UserCreationForm, UserChangeForm
from django.contrib.auth import get_user_model


UserModel = get_user_model()


class CustomAuthenticationForm(AuthenticationForm):
    username = forms.CharField(widget=forms.TextInput(attrs={'class': 'form-control', 'placeholder': UserModel.USERNAME_FIELD.capitalize()}))
    password = forms.CharField(widget=forms.PasswordInput(attrs={'class': 'form-control', 'placeholder': 'Password'}))

class CustomDateTimeInput(forms.DateTimeInput):
    def __init__(self, attrs=None):
        attrs = attrs or {}
        attrs.update({'class': 'form-control datetimepicker-input'})  # Add your custom CSS class
        super().__init__(attrs=attrs)



class BaseAdminForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        for field_name, field in self.fields.items():
            # field.label = field.label.upper()
            if isinstance(field.widget, forms.CheckboxInput):
                field.widget.attrs['class'] = field.widget.attrs.get('class', '') + ' form-check-input'
            elif isinstance(field.widget, forms.Select):
                field.widget.attrs['class'] = field.widget.attrs.get('class', '') + ' form-select'
            elif isinstance(field.widget, forms.TextInput) or isinstance(field.widget, forms.NumberInput):
                field.widget.attrs['class'] = field.widget.attrs.get('class', '') + ' form-control'
            
    class Meta:
        fields = '__all__'



class MingoUserAddForm(UserCreationForm):
    class Meta:
        model = UserModel
        fields = ('username', 'email', 'password1', 'password2')

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        for field_name, field in self.fields.items():
            field.widget.attrs.update({'class': 'form-control'})
        
class MingoUserChangeForm(UserChangeForm):
    class Meta:
        model = UserModel
        fields = '__all__'
    
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        for field_name, field in self.fields.items():
            if isinstance(field.widget, forms.TextInput):
                field.widget.attrs['class'] = field.widget.attrs.get('class', '') + ' form-control'
            if isinstance(field.widget, forms.EmailInput):
                field.widget.attrs['class'] = field.widget.attrs.get('class', '') + ' form-control'
            if isinstance(field.widget, forms.CheckboxInput):
                field.widget.attrs['class'] = field.widget.attrs.get('class', '') + ' form-check-input'
            




from django.shortcuts import render, redirect
from django.http import JsonResponse
from django.contrib.auth import logout
# Create your views here.

def logout_view(request):
    logout(request)
    return redirect('/admin/logout/')

def theme_change(request):
    theme = request.GET.get('theme')
    if theme == 'dark':
        request.session['theme'] = 'dark'
        return JsonResponse({'success': True, 'theme': 'dark'})
    elif theme == 'light':
        request.session['theme'] = 'light'
        return JsonResponse({'success': True, 'theme': 'light'})


from django import template
from django.conf import settings

register = template.Library()

@register.simple_tag
def get_settings(key):
    mingo_defined = hasattr(settings, 'MINGO_SETTINGS')
    if mingo_defined:
        mingo = settings.MINGO_SETTINGS
        return mingo.get(key, '')
    else: 
        return None


from django.conf import settings

def mingo_theme(request):
    mingo_defined = hasattr(settings, 'MINGO_SETTINGS')
    if mingo_defined:
        mingo = settings.MINGO_SETTINGS
        default_theme = mingo['theme']
    else:
        default_theme = 'light'

    return {
        'default_theme': default_theme,
    }



from django import template
from django.apps import apps

register = template.Library()

@register.filter
def get_app_icon(app):
    app_config = apps.get_app_config(app['app_label'])
    return getattr(app_config, 'icon', 'ki-home-1')

# @register.filter
# def match_app_url(request, app):
#     current_url = request.build_absolute_uri()
#     if 



from setuptools import setup, find_packages

setup(
    name='django-mingo',
    version='1.0.1',
    packages=find_packages(),
    include_package_data=True,
    license='MIT License',
    description='Customize your django admin panel.',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    author='Rupam Dhara',
    author_email='thedevgenius83@gmail.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Programming Language :: Python :: 3.10',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    install_requires=[
        'Django>=4.0',  # adjust the version as needed
        # add any other dependencies here
    ],
)
